.defaultSkin textarea {
  width: 100%;
  height: 100%;
  color: #000;
  background: #fff;
  font-size: 1.2em;
}

#content .document h1 {
  margin: 15px 0 3px 0;
}

#content .document h2 {
  margin: 12px 0 3px 9px;
}

#content .document h3 {
  margin: 9px 0 3px 18px;
  font-style: italic;
}

#content .document h4 {
  margin: 9px 0 3px 27px;
}

#content .document a:link,
#content .document a:hover,
#content .document a:visited {
  text-decoration: none;
}

#content .document h1 a,
#content .document h2 a,
#content .document h3 a {
  color: #000;
}

#content .document ul,
#content .document ol {
  padding-left: 40px;
}

#content .document li {
  margin-top: 1px;
  margin-bottom: 1px;
}

#content .document blockquote {
  margin-left: 20px;
}

#content .contents.topic {
  display: table;
  margin: 3px;
  border: 1px solid #ccc;
  background: #eec;
  padding: 0 3px;
}

#content .contents.topic ul ul {
  padding-left: 20px;
}

.topic-title.first {
  font-size: 1.1em;
  font-weight: bold;
}

.figure {
  display: table;
  padding: 1em;
  border: 1px solid #999;
  background: #ffe;
}

.reference.external:link,
.reference.external:visited {
  padding-left: 18px;
  color: blue;
  background: url(../icons/16x16/website.png) top left no-repeat;
}

.reference.internal:link,
.reference.internal:visited {
  color: blue;
}

.reference.wiki:link,
.reference.wiki:visited {
  color: green;
}

.reference.nowiki:link,
.reference.nowiki:visited {
  color: red;
}

.literal-block {
  padding: 1em;
  border: 1px dashed #729fcf;
  background: #fafafa;
}

.problematic {
  background-color: red;
}

.system-message {
  background-color: #fdd;
}

.system-message-title {
  font-weight: bold;
}

table.docutils {
  border-collapse: collapse;
  border: 2px solid #cacecf;
}

table.docutils td {
  border-bottom: 1px solid #bec1c2;
  padding: 1px 5px;
  margin: 0 5px;
}

table.docutils.field-list {
  border: none;
}

table.docutils.field-list td {
  border: none;
}
