.tracker_issue_comment_title {
  background-color: #ddddff;
  font-weight: bold;
}

.tracker_select {
  width: 97%;
}

.com {
  font-family: "Arial", "DejaVu Sans", sans-serif;
}

.tracker_product {
  color: #CC0000;
  font-style: italic;
}

.tracker_resource {
  color: #0A4A0A;
}
