tinyMCE.addI18n('zh.style_dlg',{
title:"\u7F16\u8F91 CSS \u6837\u5F0F\u8868",
apply:"\u5957\u7528",
text_tab:"\u6587\u5B57",
background_tab:"\u80CC\u666F",
block_tab:"\u533A\u5757",
box_tab:"\u65B9\u5757",
border_tab:"\u8FB9\u6846",
list_tab:"\u8868\u5217",
positioning_tab:"\u4F4D\u7F6E",
text_props:"\u6587\u5B57",
text_font:"\u5B57\u578B",
text_size:"\u5927\u5C0F",
text_weight:"\u5BBD",
text_style:"\u6837\u5F0F",
text_variant:"\u53D8\u4F53",
text_lineheight:"\u884C\u9AD8",
text_case:"\u5B57\u578B",
text_color:"\u989C\u8272",
text_decoration:"\u88C5\u9970",
text_overline:"\u9876\u7EBF",
text_underline:"\u5E95\u7EBF",
text_striketrough:"\u5220\u9664\u7EBF",
text_blink:"\u95EA\u70C1",
text_none:"\u65E0",
background_color:"\u80CC\u666F\u989C\u8272",
background_image:"\u80CC\u666F\u56FE\u6863",
background_repeat:"\u91CD\u8907",
background_attachment:"\u9644\u4EF6",
background_hpos:"\u6C34\u51C6\u4F4D\u7F6E",
background_vpos:"\u5782\u76F4\u4F4D\u7F6E",
block_wordspacing:"\u8BCD\u95F4\u8DDD",
block_letterspacing:"\u5B57\u6BCD\u95F4\u8DDD",
block_vertical_alignment:"\u5782\u76F4\u5BF9\u9F50\u65B9\u5F0F",
block_text_align:"\u6587\u5B57\u5BF9\u9F50",
block_text_indent:"\u6587\u5B57\u7F29\u6392",
block_whitespace:"\u7A7A\u683C",
block_display:"\u663E\u793A",
box_width:"\u5BBD",
box_height:"\u9AD8",
box_float:"\u6D6E\u52A8",
box_clear:"\u6E05\u9664",
padding:"\u5185\u8FB9\u8DDD",
same:"\u5168\u90E8\u76F8\u540C",
top:"\u9876\u90E8",
right:"\u53F3\u4FA7",
bottom:"\u5E95\u90E8",
left:"\u5DE6\u4FA7",
margin:"\u8FB9\u754C",
style:"\u6837\u5F0F",
width:"\u5BBD",
height:"\u9AD8",
color:"\u989C\u8272",
list_type:"\u7C7B\u578B",
bullet_image:"\u5217\u5355\u56FE\u6863",
position:"\u4F4D\u7F6E",
positioning_type:"\u7C7B\u578B",
visibility:"\u53EF\u663E\u793A",
zindex:"Z-\u5EA7\u6807",
overflow:"\u6EA2\u51FA",
placement:"\u4F48\u7F6E",
clip:"\u526A\u8F91"
});
