
h1 { font-size: 1.4em; }
h2 { font-size: 1.3em; }
h3 { font-size: 1.2em; }
h4 { font-size: 1.1em; }
h5 { font-size: 1.1em; }
h6 { font-size: 1em; }

a { color: #3465a4; }

legend {
  font-weight: bold;
}


/**************************************************************************
 * ui/aruni/template.xhtml.en
 *************************************************************************/

#header {
  text-align: right;
  background: #729fcf url(images/logo.png) left bottom no-repeat;
  color: #fff;
  font-size: 1.2em;
  margin-bottom: 10px;
  padding: 6px 10px 3px 25px;
}

#header a {
  background-position: left center;
  background-repeat: no-repeat;
  color: white;
  font-size: 1.2em;
  margin: 0px 0px 0px 0px;
  padding: 4px 2px 4px 2px;
  text-decoration: none;
}

#header a img {
  vertical-align: middle;
}

#header a:hover {
  text-decoration: underline;
}

#header #languages {
  margin-bottom: 12px;
}

#header #languages a {
  font-size: 1.1em;
}

#header #languages a.selected {
  font-weight: bold;
}

#header #top_menu a {
  margin: 0px 0px 0px 10px;
  padding: 4px 0px 4px 19px;
}

#header a#top_menu_login {
  background-image: url(../icons/16x16/lock.png);
}

#header a#top_menu_register {
  background-image: url(../icons/16x16/card.png);
}

#header a#top_menu_home {
  background-image: url(../icons/16x16/action_home.png);
}

#header a#top_menu_profile {
  background-image: url(../icons/16x16/action_home.png);
}

#header a#top_menu_logout {
  background-image: url(../icons/16x16/action_logout.png);
}

#header a#top_menu_search {
  background-image: url(../icons/16x16/search.png);
}

#header a#top_menu_contact {
  background-image: url(../icons/16x16/mail.png);
}

#content {
  width: 100%;
}

#right {
  width: 210px;
}

#footer {
  text-align: center;
  border-top: 1px solid #729fcf;
  font-size: 1.0em;
  margin: 35px auto 0 auto;
  padding: 3px;
  width: 500px;
}

#footer a {
  margin-left: 20px;
  margin-right: 20px;
  text-decoration: none;
}

#footer a:hover {
  text-decoration: underline;
}


/**************************************************************************
 * ui/aruni/template.xhtml.en
 * ui/html/addlink.xml.en
 * ui/html/addimage.xml.en
 *************************************************************************/

#body {
  padding: 10px;
  font-size: 1.4em;
  min-height: 280px;
}

#content h1, #content h2, #content h3, #content h4, #content h5, #content h6 {
  border-bottom: 1px solid #d3d7cf;
}

#content ol li { list-style-type: decimal; }
#content ul li { list-style-type: square; }

#content form fieldset {
  border-bottom: 1px solid #d3d7cf;
}

#content form legend {
  font-size: 0.9em;
}

#content form dd { padding-bottom: 5px; }
#content form p { clear: both; }

