#forum_thread_list th {
  text-align: left;
}

p.even, p.odd {
  margin: 5px;
  padding: 5px;
}

.odd {
  background-color: #ededed;
}

.forum_message_info {
  color: #fff;
}

.even .forum_message_info {
  background-color: #c17d11;
}

.odd .forum_message_info {
  background-color: #729fcf;
}

.forum_thread_title {
  font-size: 1.2em;
  font-weight: bold;
}

.forum_thread_author {
  font-weight: bold;
}

.forum_message_author {
  font-weight: bold;
}

