
/* Selector */

#cal_monthly_view #selector,
#cal_weekly_view #selector,
#cal_daily_view #selector {
  margin: 5px 0px 5px 0px;
  font-size:18px;
}

#cal_monthly_view #selector ul,
#cal_weekly_view #selector ul,
#cal_daily_view #selector ul {
  list-style:none;
  display:inline;
}

#cal_monthly_view #selector li,
#cal_weekly_view #selector li,
#cal_daily_view #selector li {
  display: inline;
  padding: 26px;
}

#cal_monthly_view #selector li a,
#cal_weekly_view #selector li a,
#cal_daily_view #selector li a {
  text-decoration:none;
  color: #cc0000;
}

/* ******** */

#cal_monthly_view table, #cal_weekly_view table {
  border-spacing: 1px;
}
#cal_monthly_view td, #cal_weekly_view td {
  background: #e9b96e;
}

#cal_monthly_view .cal_today, #cal_weekly_view .cal_today,
#cal_daily_view .cal_today {
  margin-left: 2px;
  color: #3465a4;
  font-size: 12px;
  font-style: italic;
}

#cal_monthly_view th, #cal_weekly_view th {
  background: #cc0000;
  color: #fce94f;
  font-weight: bold;
}

#cal_weekly_view th.time {
  font-size: 0.9em;
}

#cal_weekly_view th.header {
  width: 110px;
}

#cal_weekly_view th.cal_day_selected {
  width: 90px;
}

#cal_monthly_view .month {
  font-weight: bold;
  color: #cc0000;
  padding: 2px 2px 0 3px;
}

#cal_monthly_view .day,
#cal_weekly_view .day {
  height: 70px;
}

#cal_monthly_view .bold {
  font-weight: bold;
}

#cal_monthly_view .cal_day_selected, #cal_weekly_view .cal_day_selected {
  color: #cc0000;
  background: #fce94f;
}

#cal_monthly_view .cal_day_selected {
  padding: 1px;
  font-weight: bold;
}

#cal_monthly_view .add_event, #cal_weekly_view .add_event,
#cal_daily_view .add_event {
  color: #ce5c00;
  font-size:11px;
  vertical-align: top;
  text-decoration: none;
}

#cal_monthly_view .add_event img, #cal_weekly_view .add_event img {
  vertical-align: bottom;
}

#cal_monthly_view .event dd , #cal_weekly_view .event dd {
  font-size: 1em;
  margin-left: 5px;
}

#cal_monthly_view .event .CONFIRMED,
#cal_monthly_view .CONFIRMED a,
#cal_weekly_view .CONFIRMED {
  text-decoration: none;
  color: #4a0000;
}
#cal_monthly_view .event .TENTATIVE,
#cal_monthly_view .TENTATIVE a,
#cal_weekly_view .TENTATIVE {
  text-decoration: none;
  color: #cc0000;
}
#cal_monthly_view .event .CANCELLED,
#cal_monthly_view .event .CANCELLED a,
#cal_weekly_view .CANCELLED {
  color: #8f5902;
}
#cal_monthly_view .event .CANCELLED,
#cal_monthly_view .event .CANCELLED a,
#cal_weekly_view .CANCELLED {
  text-decoration: line-through;
}

#cal_monthly_view .event .time , #cal_weekly_view .event .time {
  font-size: 0.7em;
}

#cal_monthly_view .event span, #cal_weekly_view .event span {
  margin: 0;
}

table.timetable {
  border: 1px solid #ffffff;
}

/* COLORS for events background */

#cal_monthly_view table.event,
#cal_weekly_view #full_day_events table.event {
 border-collapse: collapse;
 width: 100%;
}

#cal_monthly_view table.event td,
#cal_weekly_view #full_day_events table.event td {
 border-top: 1px solid white;
 border-bottom: 1px solid white;
 padding: 2px;
}

#cal_weekly_view a span {
 padding: 2px;
/*  color: #002090; # override status color */
}

#cal_monthly_view .event .color0 td, #cal_weekly_view td.color0 {
 background-color: #BCA1A1;
}

#cal_monthly_view .event .color1 td, #cal_weekly_view td.color1 {
 background-color: #A1BCA1;
}

#cal_monthly_view .event .color2 td, #cal_weekly_view td.color2 {
 background-color: #A1A1BC;
}

#cal_monthly_view .event .color3 td, #cal_weekly_view td.color3 {
 background-color: #90A5A5;
}

#cal_monthly_view .event .color4 td, #cal_weekly_view td.color4 {
 background-color: #A5A590;
}


/**************************************************************************
 * daily_view
 **************************************************************************/
#cal_daily_view #attributes {
  margin: 5px 0 8px 0;
}

#cal_daily_view #attributes span {
  margin: 0 8px 0 0;
}

#cal_daily_view table {
  border-collapse: collapse;
  border-spacing: 1px;
}

#cal_daily_view td {
  border: 1px solid #000;
}

#cal_daily_view .header_columns {
  border-top: 1px solid black;
  font-size: 11px;
  font-weight: bold;
}

#cal_daily_view .header_columns td {
  padding: 6px;
  background: #e9b96e;
}

#cal_daily_view .header_columns th a {
  font-weight: bold;
  text-decoration: none;
}

#cal_daily_view .header_timetables,
#cal_daily_view .header_timetables * {
  border: none;
}

#cal_daily_view .header_timetables td {
  color: #fce94f;
  background: #cc0000;
  font-weight: bold;
  width: 40px;
}

#cal_daily_view .header_timetables span {
  margin-left: -15px; /* See below */
  font-size: 9px;
  font-weight: bold;
  text-align: left;
}

/* Prevent IE from eating the time outside the box */
* html #cal_daily_view .header_timetables span {
  margin-left: 0;
}

#cal_daily_view .evt_value {
  font-size: 12px;
  text-decoration: none;
  color: #003180;
}
