tinyMCE.addI18n('zh.simple',{
bold_desc:"\u7C97\u4F53 (Ctrl+B)",
italic_desc:"\u659C\u4F53 (Ctrl+I)",
underline_desc:"\u5E95\u7EBF (Ctrl+U)",
striketrough_desc:"\u5220\u9664\u7EBF",
bullist_desc:"\u4E13\u6849\u65B9\u5F0F\u6E05\u5355",
numlist_desc:"\u7F16\u53F7\u65B9\u5F0F\u6E05\u5355",
undo_desc:"\u8FD8\u539F (Ctrl+Z)",
redo_desc:"\u91CD\u8907 (Ctrl+Y)",
cleanup_desc:"\u6E05\u9664\u5197\u9980\u7801"
});
